@@ -1,38 +1,40 @@
-from typing import Dict, List
-from uuid import UUID
+from typing import Dict, List, Optional
+from uuid import UUID, uuid4
+from pathlib import Path
+import os
 
 from arkparse.object_model.cryopods.cryopod import Cryopod
-from arkparse.object_model.dinos.dino import Dino
+from arkparse.object_model.dinos.dino import Dino, DinoStats
 from arkparse.object_model.dinos.tamed_baby import TamedBaby
+from arkparse.object_model.dinos.dino_ai_controller import DinoAiController
 from arkparse.object_model.dinos.baby import Baby
 from arkparse.object_model.dinos.tamed_dino import TamedDino
 from arkparse.object_model.ark_game_object import ArkGameObject
 from arkparse.object_model.misc.dino_owner import DinoOwner
-from arkparse.ftp.ark_ftp_client import ArkFtpClient
 
 from arkparse.parsing import ArkBinaryParser
 from arkparse.saves.asa_save import AsaSave
 from arkparse.parsing import GameObjectReaderConfiguration
-from arkparse.parsing.struct.actor_transform import MapCoords
+from arkparse.parsing.struct.actor_transform import MapCoords, ActorTransform
 from arkparse.enums import ArkMap, ArkStat
-from arkparse.utils import TEMP_FILES_DIR
+from arkparse.utils import TEMP_FILES_DIR, ImportFile
 from arkparse.logging import ArkSaveLogger
-
-from importlib.resources import files
+from arkparse.classes.dinos import Dinos
+from arkparse.object_model.misc.inventory import Inventory
+from arkparse.object_model.misc.inventory_item import InventoryItem
 
 class DinoApi:
+    _DEFAULT_CONFIG = GameObjectReaderConfiguration(
+        blueprint_name_filter=lambda name: \
+            name is not None and \
+                (("Dinos/" in name and "_Character_" in name) or \
+                ("PrimalItem_WeaponEmptyCryopod_C" in name)))
+
     def __init__(self, save: AsaSave):
         self.save = save
         self.all_objects = None
         self.parsed_dinos: Dict[UUID, Dino] = {}
         self.parsed_cryopods: Dict[UUID, Cryopod] = {}
-        package = 'arkparse.assets'
-        try:
-            with open(files(package) / f'tamable_dinos.txt', 'r', encoding='utf-8') as f:
-                self.__tamables = set(line.strip() for line in f if line.strip())
-        except FileNotFoundError:
-            self.__tamables = set()
-
     def get_all_objects(self, config: GameObjectReaderConfiguration = None) -> Dict[UUID, ArkGameObject]:
         reuse = False
 
@@ -41,11 +43,7 @@ class DinoApi:
             if self.all_objects is not None:
                 return self.all_objects
 
-            config = GameObjectReaderConfiguration(
-                blueprint_name_filter=lambda name: \
-                    name is not None and \
-                        (("Dinos/" in name and "_Character_" in name) or \
-                        ("PrimalItem_WeaponEmptyCryopod_C" in name)))
+            config = self._DEFAULT_CONFIG
 
         objects = self.save.get_game_objects(config)
         
@@ -54,7 +52,7 @@ class DinoApi:
 
         return objects
     
-    def get_by_uuid(self, uuid: UUID) -> Dino:
+    def get_by_uuid(self, uuid: UUID) -> Optional[Dino]:
         object = self.save.get_game_object_by_id(uuid)
 
         if object is None:
@@ -83,66 +81,71 @@ class DinoApi:
 
         ArkSaveLogger.api_log(f"Found {len(objects)} dinos, parsing them... (and retrieving inventories)")
         for key, obj in objects.items():
-            dino = None
-            if "Dinos/" in obj.blueprint and "_Character_" in obj.blueprint:
-                is_tamed = obj.get_property_value("TamedTimeStamp") is not None
-                is_baby = obj.get_property_value("bIsBaby", False)
 
-                if obj.uuid in self.parsed_dinos:
-                    if is_tamed and include_tamed:
-                        if is_baby and include_babies:
-                            dino = self.parsed_dinos[obj.uuid]
-                        else:
-                            dino = self.parsed_dinos[obj.uuid]
-                    elif not is_tamed and include_wild:
-                        if is_baby and include_babies:
-                            dino = self.parsed_dinos[obj.uuid]
-                        else:
-                            dino = self.parsed_dinos[obj.uuid]
-                elif is_tamed and include_tamed:
-                    if is_baby and include_babies:
-                        dino = TamedBaby(obj.uuid, save=self.save)
-                    else:
-                        dino = TamedDino(obj.uuid, save=self.save)
-                    self.parsed_dinos[obj.uuid] = dino
-                elif include_wild and not is_tamed:
-                    if is_baby and include_babies:
-                        dino = Baby(obj.uuid, save=self.save)
-                    else:
-                        dino = Dino(obj.uuid, save=self.save)
-                    self.parsed_dinos[obj.uuid] = dino
+            try:
+                dino = None
+                if "Dinos/" in obj.blueprint and "_Character_" in obj.blueprint:
+                    is_tamed = obj.get_property_value("TamedTimeStamp") is not None
+                    is_baby = obj.get_property_value("bIsBaby", False)
 
-            elif "PrimalItem_WeaponEmptyCryopod_C" in obj.blueprint and include_cryos and include_tamed:
-                if not obj.get_property_value("bIsEngram", default=False):
-                    if obj.uuid in self.parsed_cryopods:
-                        is_baby = self.parsed_cryopods[obj.uuid].dino is not None and isinstance(self.parsed_cryopods[obj.uuid].dino, TamedBaby)
+                    if obj.uuid in self.parsed_dinos:
+                        if is_tamed and include_tamed:
+                            if is_baby and include_babies:
+                                dino = self.parsed_dinos[obj.uuid]
+                            else:
+                                dino = self.parsed_dinos[obj.uuid]
+                        elif not is_tamed and include_wild:
+                            if is_baby and include_babies:
+                                dino = self.parsed_dinos[obj.uuid]
+                            else:
+                                dino = self.parsed_dinos[obj.uuid]
+                    elif is_tamed and include_tamed:
                         if is_baby and include_babies:
-                            dino = self.parsed_cryopods[obj.uuid].dino
+                            dino = TamedBaby(obj.uuid, save=self.save)
                         else:
-                            dino = self.parsed_cryopods[obj.uuid].dino
-                    else:
-                        try:
-                            cryopod = Cryopod(obj.uuid, save=self.save)
-                            self.parsed_cryopods[obj.uuid] = cryopod
-                            if cryopod.dino is not None:
-                                dino = cryopod.dino
-                                dino.is_cryopodded = True
-                        except Exception as e:
-                            if "Unsupported embedded data version" in str(e):
-                                ArkSaveLogger.warning_log(f"Skipping cryopod {obj.uuid} due to unsupported embedded data version (pre Unreal 5.5)")
-                                continue
-                            ArkSaveLogger.set_log_level(ArkSaveLogger.LogTypes.PARSER, True)
-                            parser = ArkBinaryParser(self.save.get_game_obj_binary(obj.uuid), self.save.save_context)
-                            cryopod = Cryopod(obj.uuid, parser)
-                            ArkSaveLogger.set_log_level(ArkSaveLogger.LogTypes.PARSER, False)
-                            ArkSaveLogger.error_log(f"Error parsing cryopod {obj.uuid}: {e}")
+                            dino = TamedDino(obj.uuid, save=self.save)
+                        self.parsed_dinos[obj.uuid] = dino
+                    elif include_wild and not is_tamed:
+                        if is_baby and include_babies:
+                            dino = Baby(obj.uuid, save=self.save)
+                        else:
+                            dino = Dino(obj.uuid, save=self.save)
+                        self.parsed_dinos[obj.uuid] = dino
 
-                            if ArkSaveLogger._allow_invalid_objects:
-                                continue
-                            raise e
-            
-            if dino is not None:
-                dinos[key] = dino
+                elif "PrimalItem_WeaponEmptyCryopod_C" in obj.blueprint and include_cryos and include_tamed:
+                    if not obj.get_property_value("bIsEngram", default=False):
+                        if obj.uuid in self.parsed_cryopods:
+                            is_baby = self.parsed_cryopods[obj.uuid].dino is not None and isinstance(self.parsed_cryopods[obj.uuid].dino, TamedBaby)
+                            if is_baby and include_babies:
+                                dino = self.parsed_cryopods[obj.uuid].dino
+                            else:
+                                dino = self.parsed_cryopods[obj.uuid].dino
+                        else:
+                            try:
+                                cryopod = Cryopod(obj.uuid, save=self.save)
+                                self.parsed_cryopods[obj.uuid] = cryopod
+                                if cryopod.dino is not None:
+                                    dino = cryopod.dino
+                                    dino.is_cryopodded = True
+                            except Exception as e:
+                                if "Unsupported embedded data version" in str(e):
+                                    ArkSaveLogger.warning_log(f"Skipping cryopod {obj.uuid} due to unsupported embedded data version (pre Unreal 5.5)")
+                                    continue
+                                ArkSaveLogger.set_log_level(ArkSaveLogger.LogTypes.PARSER, True)
+                                cryopod = Cryopod(obj.uuid, save=self.save)
+                                ArkSaveLogger.set_log_level(ArkSaveLogger.LogTypes.PARSER, False)
+                                ArkSaveLogger.error_log(f"Error parsing cryopod {obj.uuid}: {e}")
+                                raise e
+                            finally:
+                                ArkSaveLogger.set_log_level(ArkSaveLogger.LogTypes.PARSER, False)
+
+                if dino is not None:
+                    dinos[key] = dino
+            except Exception as e:
+                if ArkSaveLogger._allow_invalid_objects:
+                    ArkSaveLogger.error_log(f"Failed to parse dino {obj.uuid}: {e}")
+                else:
+                    raise e
 
         ArkSaveLogger.api_log(f"Parsed {len(dinos)} dinos")
 
@@ -167,10 +170,20 @@ class DinoApi:
         return self.get_all(include_cryos=False, include_tamed=False)
 
     def get_all_wild_tamables(self) -> Dict[UUID, Dino]:
-        return {key: dino for key, dino in self.get_all_wild().items() if dino.get_short_name() + "_C" in self.__tamables}
+        return {key: dino for key, dino in self.get_all_wild().items() if dino.get_short_name() + "_C" not in Dinos.non_tameable.all_bps}
     
-    def get_all_tamed(self, include_cryopodded = True) -> Dict[UUID, TamedDino]:
-        return self.get_all(include_cryos=include_cryopodded, include_wild=False, include_tamed=True)
+    def get_all_tamed(self, include_cryopodded = True, only_cryopodded = False) -> Dict[UUID, TamedDino]:
+        all = self.get_all(include_cryos=include_cryopodded, include_wild=False, include_tamed=True)
+
+        if only_cryopodded:
+            tamed = {key: dino for key, dino in all.items() if isinstance(dino, TamedDino) and dino.cryopod is not None}
+        else:
+            tamed = {key: dino for key, dino in all.items() if isinstance(dino, TamedDino)}
+
+        if include_cryopodded:
+            return tamed
+        else:
+            return {key: dino for key, dino in tamed.items() if dino.cryopod is None}
 
     def get_all_babies(self, include_tamed: bool = True, include_cryopodded: bool = True, include_wild: bool = False) -> Dict[UUID, TamedBaby]:
         dinos = self.get_all(include_cryos=include_cryopodded, include_wild=include_wild, include_tamed=include_tamed)
@@ -180,14 +193,17 @@ class DinoApi:
         return babies
     
     def get_all_in_cryopod(self) -> Dict[UUID, TamedDino]:
-        return self.get_all(include_cryos=True, include_wild=False, include_tamed=False)
-    
-    def get_all_by_class(self, class_names: List[str]) -> Dict[UUID, Dino]:
+        tamed = self.get_all_tamed(include_cryopodded=True)
+        cryod = {key: dino for key, dino in tamed.items() if dino.cryopod is not None}
+
+        return cryod
+
+    def get_all_by_class(self, class_names: List[str], include_cryopodded: bool = True) -> Dict[UUID, Dino]:
         config = GameObjectReaderConfiguration(
             blueprint_name_filter=lambda name: name is not None and name in class_names
         )
 
-        dinos = self.get_all(config)
+        dinos = self.get_all(config, include_cryos=include_cryopodded)
         class_dinos = {k: v for k, v in dinos.items() if v.object.blueprint in class_names}
 
         return class_dinos
@@ -197,9 +213,9 @@ class DinoApi:
         wild_dinos = {k: v for k, v in dinos.items() if not isinstance(v, TamedDino)}
 
         return wild_dinos
-    
-    def get_all_tamed_by_class(self, class_name: List[str]) -> Dict[UUID, TamedDino]:
-        dinos = self.get_all_by_class(class_name)
+
+    def get_all_tamed_by_class(self, class_name: List[str], include_cryopodded: bool = True) -> Dict[UUID, TamedDino]:
+        dinos = self.get_all_by_class(class_name, include_cryopodded=include_cryopodded)
         tamed_dinos = {k: v for k, v in dinos.items() if isinstance(v, TamedDino)}
 
         return tamed_dinos
@@ -353,18 +369,12 @@ class DinoApi:
 
         return cryopodded
     
-    def modify_dinos(self, dinos: Dict[UUID, TamedDino], new_owner: DinoOwner = None, ftp_client: ArkFtpClient = None):
+    def modify_dinos(self, dinos: Dict[UUID, TamedDino], new_owner: DinoOwner = None):
         for key, dino in dinos.items():
             if new_owner is not None:
                 dino.owner.replace_with(new_owner, dino.binary)
                 dino.update_binary()
 
-        if ftp_client is not None:
-            self.save.store_db(TEMP_FILES_DIR / "sapi_temp_save.ark")
-            ftp_client.connect()
-            ftp_client.upload_save_file(TEMP_FILES_DIR / "sapi_temp_save.ark")
-            ftp_client.close()
-
     def create_heatmap(self, map: ArkMap, resolution: int = 100, dinos: Dict[UUID, TamedDino] = None, classes: List[str] = None, owner: DinoOwner = None, only_tamed: bool = False):
         import math
         import numpy as np
@@ -392,15 +402,16 @@ class DinoApi:
 
         return np.array(heatmap)
     
-    def get_best_dino_for_stat(self, classes: List[str] = None, stat: ArkStat = None, only_tamed: bool = False, only_untamed: bool = False, base_stat: bool = False, mutated_stat=False) -> (Dino, int, ArkStat):
+    
+    def get_best_dino_for_stat(self, classes: List[str] = None, stat: ArkStat = None, only_tamed: bool = False, only_untamed: bool = False, base_stat: bool = False, mutated_stat=False, level_upper_bound=None) -> (Dino, int, ArkStat):
         if only_tamed and only_untamed:
             raise ValueError("Cannot specify both only_tamed and only_untamed")
         
         if mutated_stat and base_stat:
             raise ValueError("Cannot specify both base_stat and base_mutated_stat")
         
-        if classes is not None:
-            dinos = self.get_all_filtered(class_names=classes, include_cryopodded=True)
+        if classes is not None or level_upper_bound is not None:
+            dinos = self.get_all_filtered(class_names=classes, include_cryopodded=True, level_upper_bound=level_upper_bound)
         else:
             dinos = self.get_all()
 
@@ -441,4 +452,122 @@ class DinoApi:
                 return obj
 
         return None
+    
+    def __get_all_files_from_dir_recursive(self, dir_path: Path) -> list[ImportFile]:
+        out = []
+        base_file = None
+        for root, _, files in os.walk(dir_path):
+            for file in files:
+                file_path = Path(root) / Path(file)
+                if file_path.name.endswith(".bin") or file_path.name.startswith("loc_"):
+                    out.append(ImportFile(str(file_path)))
+        return out
 
+    def import_dino(self, path: Path, location: ActorTransform = None) -> Dino | TamedDino:
+        uuid_translation_map = {}
+
+        def replace_uuids(uuid_map: Dict[UUID, UUID], bytes_: bytes):
+            for uuid in uuid_map:
+                new_bytes = uuid_map[uuid].bytes            
+                old_bytes = uuid.bytes
+                bytes_ = bytes_.replace(old_bytes, new_bytes)
+                # print(f"Replacing {uuid} with {uuid_map[uuid]}")
+            return bytes_
+
+        actor_transforms: Dict[UUID, ActorTransform] = {}
+        files: List[ImportFile] = self.__get_all_files_from_dir_recursive(path)
+
+        # assign new uuids to all
+        for file in files:
+            uuid_translation_map[file.uuid] = uuid4()
+
+        # Assign new uuids to all actor transforms and add them to the database
+        new_actor_transforms: bytes = bytes()
+        for file in files:
+            if file.type == "loc":
+                new_uuid: UUID = uuid_translation_map[file.uuid]
+                actor_transforms[new_uuid] = ActorTransform(from_json=Path(file.path))
+                new_actor_transforms += new_uuid.bytes + actor_transforms[new_uuid].to_bytes()
+                ArkSaveLogger.api_log(f"Added actor transform {new_uuid} to DB")
+        self.save.add_actor_transforms(new_actor_transforms)
+        # Update actor transforms in save context
+        self.save.read_actor_locations()
+
+        # get all inventory items and add them to DB
+        for file in files:
+            if file.type == "itm":
+                new_uuid = uuid_translation_map[file.uuid]
+                parser = ArkBinaryParser(file.bytes, self.save.save_context)
+                parser.byte_buffer = replace_uuids(uuid_translation_map, parser.byte_buffer)
+                parser.replace_name_ids(file.names, self.save)
+                self.save.add_obj_to_db(new_uuid, parser.byte_buffer)
+                item = InventoryItem(uuid=new_uuid, save=self.save)
+                item.reidentify(new_uuid)
+                ArkSaveLogger.api_log(f"Added inventory item {item.uuid} to DB")
+
+        # Get inventory and add to DB
+        inventory = None
+        for file in files:
+            if file.type == "inv":
+                new_uuid = uuid_translation_map[file.uuid]
+                parser = ArkBinaryParser(file.bytes, self.save.save_context)
+                parser.byte_buffer = replace_uuids(uuid_translation_map, parser.byte_buffer)
+                parser.replace_name_ids(file.names, self.save)
+                self.save.add_obj_to_db(new_uuid, parser.byte_buffer)
+                inventory = Inventory(uuid=new_uuid, save=self.save)
+                inventory.reidentify(new_uuid)
+                ArkSaveLogger.api_log(f"Added inventory {inventory.uuid} to DB")
+
+        # Get status and add to DB
+        for file in files:
+            if file.type == "status":
+                new_uuid = uuid_translation_map[file.uuid]
+                parser = ArkBinaryParser(file.bytes, self.save.save_context)
+                parser.byte_buffer = replace_uuids(uuid_translation_map, parser.byte_buffer)
+                parser.replace_name_ids(file.names, self.save)
+                self.save.add_obj_to_db(new_uuid, parser.byte_buffer)
+                stats = DinoStats(uuid=new_uuid, save=self.save)
+                stats.reidentify(new_uuid)
+                ArkSaveLogger.api_log(f"Added dino stats {stats.uuid} to DB")
+
+        # Get AI controller and add to DB
+        for file in files:
+            if file.type == "ai":
+                new_uuid = uuid_translation_map[file.uuid]
+                parser = ArkBinaryParser(file.bytes, self.save.save_context)
+                parser.byte_buffer = replace_uuids(uuid_translation_map, parser.byte_buffer)
+                parser.replace_name_ids(file.names, self.save)
+                self.save.add_obj_to_db(new_uuid, parser.byte_buffer)
+                ai_controller = DinoAiController(uuid=new_uuid, save=self.save)
+                ai_controller.reidentify(new_uuid)
+                ArkSaveLogger.api_log(f"Added AI controller {ai_controller.uuid} to DB")
+
+        # Get dino and add to DB
+        for file in files:
+            if file.type == "obj":
+                new_uuid = uuid_translation_map[file.uuid]
+                parser = ArkBinaryParser(file.bytes, self.save.save_context)
+                parser.byte_buffer = replace_uuids(uuid_translation_map, parser.byte_buffer)
+                parser.replace_name_ids(file.names, self.save)
+                self.save.add_obj_to_db(new_uuid, parser.byte_buffer)
+                dino = Dino(uuid=new_uuid, save=self.save)
+                dino.reidentify(new_uuid)
+                ArkSaveLogger.api_log(f"Added dino {dino.uuid} to DB")
+
+                if location is not None:
+                    dino.set_location(location)
+
+                ArkSaveLogger.api_log(f"Replacing name \"{dino.stats.object.names[1]}\" with \"{dino.object.names[0]}\"")
+                dino.stats.replace_name_at_index_with(1, dino.object.names[0])
+                dino.stats.update_binary()
+                
+                if inventory is not None:
+                    ArkSaveLogger.api_log(f"Replacing inventory name \"{inventory.object.names[1]}\" with \"{dino.object.names[0]}\"")
+                    inventory.replace_name_at_index_with(1, dino.object.names[0])
+                    inventory.update_binary()
+                    dino: TamedDino = TamedDino(uuid=new_uuid, save=self.save)
+                    return dino
+
+                return dino
+
+        raise ValueError("No dino object found in the provided files. Please ensure the directory contains valid dino files.")
\ No newline at end of file
