@@ -13,9 +13,8 @@ from .struct.ark_struct_type import ArkStructType
 from .struct.ark_dino_ancestor_entry import ArkDinoAncestorEntry
 
 from .ark_binary_parser import ArkBinaryParser
-from .ark_value_type import ArkValueType
+from arkparse.parsing.ark_value_type import ArkValueType
 from arkparse.enums.ark_enum import ArkEnumValue
-
 from arkparse.parsing.ark_property_container import ArkPropertyContainer
 from arkparse.parsing.struct.ark_color import ArkColor
 from arkparse.parsing.struct.ark_linear_color import ArkLinearColor
@@ -43,6 +42,9 @@ class ArkProperty:
         self.binary_position = 0
         self.nr_of_bytes = 0
 
+    def to_json_obj(self):
+        return { "name": self.name, "type": self.type, "value": self.value.__str__() }
+
     @staticmethod
     def read_property(byte_buffer: 'ArkBinaryParser', in_array: bool = False) -> Optional['ArkProperty']:
         key = byte_buffer.read_name()
@@ -68,6 +70,8 @@ class ArkProperty:
 
         if value_type == ArkValueType.Boolean:
             p = ArkProperty(key, value_type.name, position, 0, ArkProperty.read_property_value(value_type, byte_buffer))
+            if not in_array:
+                byte_buffer.validate_byte(0)
         elif value_type in {ArkValueType.Float, ArkValueType.Int, ArkValueType.Int8, ArkValueType.Double, ArkValueType.UInt32,
                             ArkValueType.UInt64, ArkValueType.UInt16, ArkValueType.Int16, ArkValueType.Int64,
                             ArkValueType.String, ArkValueType.Name, ArkValueType.SoftObject, ArkValueType.Object}:
@@ -176,11 +180,14 @@ class ArkProperty:
     @staticmethod
     def read_soft_object_property_value(byte_buffer: 'ArkBinaryParser') -> str:
         ArkSaveLogger.enter_struct("SfO")
-        obj_name = byte_buffer.read_name()
+        names = []
+        while byte_buffer.peek_int() != 0:
+            obj_name = byte_buffer.read_name()
+            names.append(obj_name)
         byte_buffer.validate_bytes_as_string("00 00 00 00", 4)
-        ArkSaveLogger.parser_log(f"Read soft object property {obj_name}")
+        ArkSaveLogger.parser_log(f"Read soft object property {names}")
         ArkSaveLogger.exit_struct()
-        return obj_name
+        return names
 
     @staticmethod
     def read_struct_property(byte_buffer: 'ArkBinaryParser', data_size: int, struct_type: str, in_array: bool) -> Any:       
@@ -234,9 +241,8 @@ class ArkProperty:
         properties = ArkProperty.read_struct_properties(byte_buffer)
 
         if byte_buffer.get_position() != position + data_size and not in_array:
-            if not ArkSaveLogger.suppress_warnings:
-                print("WARNING: Struct reading position mismatch for type", struct_type)
-                print(f"StructType: {struct_type}, DataSize: {data_size}, Position: {position}, CurrentPosition: {byte_buffer.get_position()}")
+            ArkSaveLogger.warning_log("Struct reading position mismatch for type", struct_type)
+            ArkSaveLogger.warning_log(f"StructType: {struct_type}, DataSize: {data_size}, Position: {position}, CurrentPosition: {byte_buffer.get_position()}")
             byte_buffer.set_position(position + data_size)
             # ArkSaveLogger.open_hex_view()
             # raise Exception("Struct reading position mismatch: [StructType: %s, DataSize: %d, Position: %d, CurrentPosition: %d]" % (struct_type, data_size, position, byte_buffer.get_position()))
@@ -311,9 +317,6 @@ class ArkProperty:
 
             if(byte_buffer.position != data_start_postion + data_size):
                 # just skip to the end of the struct if an error occurs
-                if not ArkSaveLogger.suppress_warnings:
-                    print("WARNING: Array read incorrectly, bytes left to read:", data_start_postion + data_size - byte_buffer.position)
-                    print("Skipping to the end of the struct, type:", array_content_type)
                 byte_buffer.set_position(data_start_postion + data_size)
                 # byte_buffer.find_names()
                 # ArkSaveLogger.open_hex_view()
@@ -384,7 +387,7 @@ class ArkProperty:
         elif value_type == ArkValueType.Name:
             return byte_buffer.read_name()
         elif value_type == ArkValueType.Boolean:
-            return byte_buffer.read_short() == 1
+            return byte_buffer.read_byte() == 1
         elif value_type == ArkValueType.Struct:
             return ArkProperty.read_struct_property(byte_buffer, byte_buffer.read_int(), True)
         elif value_type == ArkValueType.SoftObject:
