@@ -23,12 +23,16 @@ from arkparse.parsing.struct.ark_gigantoraptor_bonded_struct import ArkGigantora
 from arkparse.parsing.struct.ark_tracked_actor_id_category_pair_with_bool import (
     ArkTrackedActorIdCategoryPairWithBool,
 )
+from arkparse.parsing.struct.ark_tracked_actor_id_category_pair import ArkTrackedActorIdCategoryPair
 from arkparse.parsing.struct.ark_my_persistent_buff_datas import ArkMyPersistentBuffDatas
 from arkparse.parsing.struct.ark_item_net_id import ArkItemNetId
 from arkparse.parsing.struct.object_reference import ObjectReference
 from arkparse.parsing.struct.ark_struct_type import ArkStructType
 from arkparse.parsing.struct.ark_dino_ancestor_entry import ArkDinoAncestorEntry
 from arkparse.parsing.struct.ark_custom_item_data import ArkCustomItemData
+from arkparse.parsing.struct.ark_painting_key_value import ArkPaintingKeyValue
+from arkparse.parsing.struct.ark_dino_order_id import ArkDinoOrderID
+from arkparse.parsing.struct.ark_tribe_alliance import ArkTribeAlliance
 
 from arkparse.parsing.ark_property_container import ArkPropertyContainer
 from arkparse.parsing.ark_set import ArkSet
@@ -74,6 +78,7 @@ _STRUCT_READERS: Dict[ArkStructType, Callable[["ArkBinaryParser", int], Any]] =
     ArkStructType.UniqueNetIdRepl: lambda bb, ds: ArkUniqueNetIdRepl(bb),
     ArkStructType.VectorBoolPair: lambda bb, ds: ArkVectorBoolPair(bb),
     ArkStructType.ArkTrackedActorIdCategoryPairWithBool: lambda bb, ds: ArkTrackedActorIdCategoryPairWithBool(bb),
+    ArkStructType.ArkTrackedActorIdCategoryPair: lambda bb, ds: ArkTrackedActorIdCategoryPair(bb),
     ArkStructType.MyPersistentBuffDatas: lambda bb, ds: ArkMyPersistentBuffDatas(bb, ds),
     ArkStructType.ItemNetId: lambda bb, ds: ArkItemNetId(bb),
     ArkStructType.ArkDinoAncestor: lambda bb, ds: ArkDinoAncestorEntry(bb),
@@ -86,6 +91,9 @@ _STRUCT_READERS: Dict[ArkStructType, Callable[["ArkBinaryParser", int], Any]] =
     ArkStructType.ArkGeneTraitStruct: lambda bb, ds: ArkGeneTraitStruct(bb),
     ArkStructType.GachaResourceStruct: lambda bb, ds: ArkGachaResourceStruct(bb),
     ArkStructType.GigantoraptorBondedStruct: lambda bb, ds: ArkGigantoraptorBondedStruct(bb),
+    ArkStructType.ArkPaintingKeyValue: lambda bb, ds: ArkPaintingKeyValue(bb),
+    ArkStructType.ArkDinoOrderID: lambda bb, ds: ArkDinoOrderID(bb),
+    ArkStructType.ArkTribeAlliance: lambda bb, ds: ArkTribeAlliance(bb),
 }
 
 # Flags driving how a primitive value is read
@@ -157,7 +165,9 @@ class ArkProperty:
     def read_property(byte_buffer: "ArkBinaryParser", in_array: bool = False) -> Optional["ArkProperty"]:
         name_position = byte_buffer.get_position()
         value_position = 0
+        byte_buffer.save_context.generate_unknown = True
         key = byte_buffer.read_name()
+        byte_buffer.save_context.generate_unknown = False
 
         if key is None or key == "None":
             ArkSaveLogger.parser_log("Exiting struct (None marker)")
@@ -180,9 +190,10 @@ class ArkProperty:
         elif value_type == ArkValueType.Byte:
             prop, value_position = ArkProperty._read_byte_property(key, position, data_size, byte_buffer)
         elif value_type == ArkValueType.Struct:
-            byte_buffer.set_position(byte_buffer.get_position() - 4)  # V14 fix
+            byte_buffer.set_position(byte_buffer.get_position() - 8)  # V14 fix
+            nr_of_names = byte_buffer.read_uint32()
             struct_type = byte_buffer.read_name()
-            val, value_position = ArkProperty.read_struct_property(byte_buffer, data_size, struct_type, in_array)
+            val, value_position = ArkProperty.read_struct_property(byte_buffer, data_size, struct_type, in_array, nr_of_names=nr_of_names)
             prop = ArkProperty(key, value_type.name, position, 0, val)
         elif value_type == ArkValueType.Array:
             prop = ArkProperty.read_array_property(key, value_type.name, position, byte_buffer, data_size)
@@ -380,7 +391,7 @@ class ArkProperty:
             prop = ArkProperty(key, type_, position, 0, struct_array)
             if bb.position != data_start_position + data_size:
                 ArkSaveLogger.warning_log(
-                    f"WARNING: Array read incorrectly, bytes left to read: {data_start_position + data_size - bb.position}"
+                    f"Array read incorrectly, bytes left to read: {data_start_position + data_size - bb.position}"
                 )
                 ArkSaveLogger.warning_log(f"Skipping to the end of the struct, type: {array_content_type}")
                 bb.set_position(data_start_position + data_size)
@@ -429,27 +440,35 @@ class ArkProperty:
     @staticmethod
     def __read_struct_header(bb: "ArkBinaryParser", position: int = 0, in_array: bool = False, in_map: bool = False, nr_of_struct_names: int = 1) -> Tuple[int, int, bool]:
         bb.validate_uint32(1)  # V14 marker
-        for _ in range(nr_of_struct_names):
-            _new_name = bb.read_name()
-            bb.validate_uint32(0)
-        data_size = bb.read_uint32()
-        size_byte = bb.read_byte()  # V14 unknown byte
+        with log_block("StructHeader"):
+            if nr_of_struct_names > 10:
+                ArkSaveLogger.warning_log(f"Too many struct names: {nr_of_struct_names}; reverting to reading one name")
+                nr_of_struct_names = 1
+            ArkSaveLogger.parser_log(f"reading {nr_of_struct_names} names")
+            for i in range(nr_of_struct_names):
+                _new_name = bb.read_name()
+                ArkSaveLogger.parser_log(f"name {i}: {_new_name}")
+                bb.validate_uint32(0)
+            data_size = bb.read_uint32()
+            size_byte = bb.read_byte()  # V14 unknown byte
 
-        no_pos_values = [0, 8]
-        if in_array or in_map:
-            no_pos_values = []
+            no_pos_values = [0, 8]
+            if in_array or in_map:
+                no_pos_values = []
 
-        read_pos = (size_byte not in no_pos_values)
-        if read_pos:
-            position = bb.read_uint32()
+            read_pos = (size_byte not in no_pos_values)
+            if read_pos:
+                position = bb.read_uint32()
+
+            ArkSaveLogger.parser_log(f"pos byte={size_byte}, pos read={read_pos}, position={position}, data size={data_size}")
 
         return data_size, position, read_pos, size_byte
 
     @staticmethod
-    def read_struct_property(bb: "ArkBinaryParser", data_size: int, struct_type: str, in_array: bool) -> Any:
+    def read_struct_property(bb: "ArkBinaryParser", data_size: int, struct_type: str, in_array: bool, nr_of_names: int = 1) -> Any:
         if not in_array:
-            with log_block(f"S({struct_type})"):
-                data_size, _, _, _ = ArkProperty.__read_struct_header(bb)
+            with log_block(f"S({struct_type})"):            
+                data_size, _, _, _ = ArkProperty.__read_struct_header(bb, nr_of_struct_names=nr_of_names)
                 value_position = bb.get_position()
                 return ArkProperty._read_struct_body(bb, data_size, struct_type, in_array), value_position
         else:
@@ -476,6 +495,7 @@ class ArkProperty:
                 # uncomment the lines below if you want to make objects of unknown structs
                 # ArkSaveLogger.parser_log(f"Reading struct {struct_type} as array")
                 # bb.structured_print(to_default_file=True)
+                # bb.store()
                 # ArkSaveLogger.error_log(f"Unsupported struct type {struct_type} in array")
                 # ArkSaveLogger.open_hex_view(True)
                 # raise ValueError(f"Unsupported struct type {struct_type}")
@@ -523,18 +543,21 @@ class ArkProperty:
             return _SIMPLE_SPECS[value_type].reader(bb)
         if value_type == ArkValueType.Byte:
             return bb.read_unsigned_byte()
-        if value_type == ArkValueType.Struct:
-            prop, _ = ArkProperty.read_struct_property(bb, bb.read_int(), True)
-            return prop
+        # if value_type == ArkValueType.Struct:
+        #     prop, _ = ArkProperty.read_struct_property(bb, bb.read_int(), True)
+        #     return prop
         raise RuntimeError(f"Cannot read value type: {value_type} at position {bb.get_position()}")
 
     @staticmethod
     def read_soft_object_property_value(bb: "ArkBinaryParser") -> str:
         with log_block("SfO"):
-            obj_name = bb.read_name()
+            names = []
+            while bb.peek_int() != 0:
+                obj_name = bb.read_name()
+                names.append(obj_name)
             bb.validate_bytes_as_string("00 00 00 00", 4)
-            ArkSaveLogger.parser_log(f"Read soft object property {obj_name}")
-            return obj_name
+            ArkSaveLogger.parser_log(f"Read soft object property {names}")
+            return names
 
     @staticmethod
     def _fixup_if_left(bb: "ArkBinaryParser", start: int, size: int, label: str) -> None:
