@@ -8,6 +8,7 @@ from .ark_quat import ArkQuat
 from .ark_rotator import ArkRotator
 from .ark_struct_type import ArkStructType
 from .ark_tracked_actor_id_category_pair_with_bool import ArkTrackedActorIdCategoryPairWithBool
+from .ark_tracked_actor_id_category_pair import ArkTrackedActorIdCategoryPair
 from .ark_unique_net_id_repl import ArkUniqueNetIdRepl
 from .ark_vector_bool_pair import ArkVectorBoolPair
 from .ark_vector import ArkVector
@@ -15,4 +16,7 @@ from .object_reference import ObjectReference, get_uuid_reference_bytes
 from .unknown_struct import UnknownStruct
 from .ark_custom_item_data import ArkCustomItemData
 from .ark_key_value_pair import ArkKeyValuePair
+from .ark_painting_key_value import ArkPaintingKeyValue
+from .ark_dino_order_id import ArkDinoOrderID
+from .ark_tribe_alliance import ArkTribeAlliance
 
